package polaris.application;

/**
 * The version numbers
 * 
 * @author johnson
 * 
 */
public final class Version {

	/**
	 * Major version
	 */
	public final int major;

	/**
	 * Minor version
	 */
	public final int minor;

	/**
	 * Revision version
	 */
	public final int revision;

	/**
	 * Build version
	 */
	public final int build;

	/**
	 * Construct a {@link Version} object with specified version numbers.
	 * 
	 * @param major
	 *            Major version.
	 */
	public Version(int major) {
		this.major = major;
		this.minor = 0;
		this.revision = 0;
		this.build = 0;
	}

	/**
	 * Construct a {@link Version} object with specified version numbers.
	 * 
	 * @param major
	 *            Major version.
	 * @param minor
	 *            Minor version.
	 */
	public Version(int major, int minor) {
		this.major = major;
		this.minor = minor;
		this.revision = 0;
		this.build = 0;
	}

	/**
	 * Construct a {@link Version} object with specified version numbers.
	 * 
	 * @param major
	 *            Major version.
	 * @param minor
	 *            Minor version.
	 * @param revision
	 *            Revision version.
	 */
	public Version(int major, int minor, int revision) {
		this.major = major;
		this.minor = minor;
		this.revision = revision;
		this.build = 0;
	}

	/**
	 * Construct a {@link Version} object with specified version numbers.
	 * 
	 * @param major
	 *            Major version.
	 * @param minor
	 *            Minor version.
	 * @param revision
	 *            Revision version.
	 * @param build
	 *            Build version.
	 */
	public Version(int major, int minor, int revision, int build) {
		this.major = major;
		this.minor = minor;
		this.revision = revision;
		this.build = build;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;

		result = prime * result + build;
		result = prime * result + major;
		result = prime * result + minor;
		result = prime * result + revision;

		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;

		if (obj == null || getClass() != obj.getClass())
			return false;

		Version other = (Version) obj;

		if (build != other.build)
			return false;

		if (major != other.major)
			return false;

		if (minor != other.minor)
			return false;

		if (revision != other.revision)
			return false;

		return true;
	}

	@Override
	public String toString() {
		return major + "." + minor + "." + revision + " build-" + build;
	}

}
