package polaris.persistence;

import java.io.Serializable;
import java.util.List;

/**
 * Manager of persistence
 * 
 * @author johnson
 * 
 */
public interface PersistenceManager {

	/**
	 * Fetch all objects.
	 * 
	 * @param clazz
	 *            The object type.
	 * @return a list of objects.
	 */
	public <T> List<T> fetch(Class<T> clazz);

	/**
	 * Find an object by id.
	 * 
	 * @param clazz
	 *            The object type.
	 * @param id
	 *            The identifier.
	 * @return an object.
	 * @throws PersistenceException
	 */
	public <T> T find(Class<T> clazz, Serializable id)
			throws PersistenceException;

	/**
	 * Persist a specify object
	 * 
	 * @param t
	 *            an object.
	 * @throws PersistenceException
	 */
	public <T> void persist(T t) throws PersistenceException;

	/**
	 * Delete a specify object.
	 * 
	 * @param t
	 *            an object.
	 * @throws PersistenceException
	 */
	public <T> void delete(T t) throws PersistenceException;

	/**
	 * Update a specify object.
	 * 
	 * @param t
	 *            an object.
	 * @throws PersistenceException
	 */
	public <T> void update(T t) throws PersistenceException;

	/**
	 * Execute a transaction
	 * 
	 * @param transaction
	 * @throws PersistenceException
	 */
	public void execute(Transaction transaction) throws PersistenceException;

}
