package polaris.persistence;

import java.util.HashMap;
import java.util.Map;

import polaris.application.Application;

public abstract class PersistenceManagerFactory {
	private static final String FACTORY_ID = "com.cytaster.persistence.PersistenceManagerFactory";
	private static final String FACTORYIMPL_ID = "com.cytaster.persistence.PersistenceManagerFactoryImpl";

	private static final Map<Class<?>, PersistenceManagerFactory> registry = new HashMap<Class<?>, PersistenceManagerFactory>();

	protected PersistenceManagerFactory() {
	}

	public static PersistenceManagerFactory newInstance(Application app)
			throws InstantiationException, IllegalAccessException {
		String clsName = System.getProperty(FACTORY_ID, FACTORYIMPL_ID);

		try {
			Class<?> clazz = Class.forName(clsName);

			if (PersistenceManagerFactory.registry.containsKey(clazz))
				return PersistenceManagerFactory.registry.get(clazz);

			PersistenceManagerFactory factory = (PersistenceManagerFactory) clazz
					.newInstance();
			return PersistenceManagerFactory.registry.put(clazz, factory);
		} catch (ClassNotFoundException e) {
			return null;
		}
	}

	public abstract PersistenceManager newPersistenceManager();
}
