package polaris.reflect;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

/**
 * A common introspector
 * 
 * @author johnson
 * 
 * @param <T>
 *            The type of introspected object.
 */
public class GenericIntrospector<T> implements Introspector<T> {
	protected T obj;

	public GenericIntrospector(T obj) {
		this.obj = obj;
	}

	@Override
	public T get() {
		return this.obj;
	}

	@Override
	public void set(T obj) {
		this.obj = obj;
	}

	@Override
	public Object get(String name) throws IllegalAccessException {
		Class<?> clazz = this.obj.getClass();
		String attr = Character.toUpperCase(name.charAt(0)) + name.substring(1);

		try {
			Method getter = clazz.getMethod("get" + attr);
			return getter.invoke(this.obj);
		} catch (Throwable t) {
			throw new IllegalAccessException(t.getMessage());
		}
	}

	@Override
	public void set(String name, Object value) throws IllegalAccessException {
		Class<?> clazz = this.obj.getClass();
		String attr = Character.toUpperCase(name.charAt(0)) + name.substring(1);

		try {
			Method getter = clazz.getMethod("set" + attr);
			getter.invoke(this.obj, value);
		} catch (Throwable t) {
			throw new IllegalAccessException(t.getMessage());
		}
	}

	@Override
	public String[] getAttributeNames() {
		Class<?> clazz = this.obj.getClass();
		Field[] fields = clazz.getDeclaredFields();
		List<String> names = new ArrayList<String>();

		for (int i = 0; i < fields.length; i++) {
			if (this.hasAttribute(fields[i].getName())) {
				names.add(fields[i].getName());
			}
		}

		return names.toArray(new String[names.size()]);
	}

	@Override
	public boolean hasAttribute(String name) {
		Class<?> clazz = this.obj.getClass();
		String attr = Character.toUpperCase(name.charAt(0)) + name.substring(1);

		try {
			Method getter = clazz.getMethod("get" + attr);
			if (null == getter)
				return false;
		} catch (Throwable t) {
			return false;
		}

		return true;
	}

}
